-- get the addon namespace
local addon, ns = ...
local cfg = ns.cfg
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.inrange = function(spell)
	if cfg.spells[spell] then 
		if IsSpellInRange(cfg.spells[spell].name,"target")==1 then return true end
	else
		if IsSpellInRange(spell,"target")==1 then return true end
	end
	return nil
end

local onground
lib.GroundUpdate = function()
	onground=cfg.onGround
	if IsFalling() or IsFlying() then
		cfg.onGround=false
	else
		cfg.onGround=true
	end
	
	if onground~=cfg.onGround then
		cfg.Update=true
		return true
	end
	return nil
end

local shape
lib.UpdateShape = function()
	shape=cfg.shape
	cfg.shapeid=GetShapeshiftFormID()
	if cfg.shapeid and cfg.shapes[cfg.shapeid] then
		cfg.shape=cfg.shapes[cfg.shapeid]
	else 
		cfg.shape="human"
	end
	if shape~=cfg.shape then
		cfg.Update=true
		return true
	end
	return nil
end

local itemID,i
lib.UpdateSet = function()
	cfg.setitems={}
	for itemSET, _ in pairs(cfg.set) do
		cfg.setitems[itemSET]=0
		for _, itemID in pairs(cfg.set[itemSET]) do
			for i=0,19 do
				if cfg.equip[i] and cfg.equip[i]==itemID then
					cfg.setitems[itemSET]=cfg.setitems[itemSET]+1
				end
			end
		end
	end
end

lib.trinket = function(trinket)
	for _, itemID in pairs(cfg.trinket[trinket]) do
		for i=13,14 do
			if cfg.equip[i] and cfg.equip[i]==itemID then
				return true
			end
		end
	end
	return nil
end

lib.SetBonus=function(set)
	if cfg.setitems[set] then
		if cfg.setitems[set]>=4 then
			return 2
		elseif cfg.setitems[set]>=2 then
			return 1
		end
	end
	return 0
end

lib.UpdateEquip=function(slot)
	cfg.equip[slot]=GetInventoryItemID("player", slot)
end

lib.UpdateEquipAll=function()
	for i=0,19 do
		lib.UpdateEquip(i)
	end
	lib.UpdateSet()
end

local cspell
lib.SwitchMode = function()
	if cfg.mode~="dps" then
		for _,cspell in ipairs(cfg.spells_single) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or lib.SpellCasting(cspell)) then
				cfg.mode = "dps"
				cfg.plist=cfg.plistdps
				cfg.Update=true
				return
			end
		end
	end
	
	if cfg.mode~="aoe" then
		for _,cspell in ipairs(cfg.spells_aoe) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or lib.SpellCasting(cspell)) then
				cfg.mode = "aoe"
				cfg.plist=cfg.plistaoe
				cfg.Update=true
				return
			end
		end
	end
	
	if cfg.mode~="range" then
		for _,cspell in ipairs(cfg.spells_range) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or lib.SpellCasting(cspell)) then
				cfg.mode = "range"
				cfg.plist=cfg.plistrange
				cfg.Update=true
				return
			end
		end
	end
end

lib.instance=function()
	cfg.instance=select(2,IsInInstance())
	--[[
	arena - Player versus player arena
	none - Not inside an instance
	party - 5-man instance
	pvp - Player versus player battleground
	raid - Raid instance
	]]
	return cfg.instance
end

lib.ininstance=function()
	if cfg.instance=="raid" or cfg.instance=="party" then
		return true 
	else
		return nil
	end
end

lib.raidboss = function()
	if cfg.target=="worldboss" and cfg.instance=="raid" then return true end
	return nil
end

lib.UpdateTarget = function()
	cfg.target=UnitClassification("target")
	if UnitLevel("target")==-1 then cfg.target="worldboss" end
	cfg.targetGUID=UnitGUID("target")
	--[[
	elite - Elite
	normal - Normal
	minus - Small
	rare - Rare
	rareelite - Rare-Elite
	worldboss - World Boss
	]]
--	print(cfg.target.." "..cfg.instance)
	return cfg.target
end

lib.hardtarget = function()
	if cfg.target=="normal" or cfg.target=="minus" then return nil end
	return true
end

lib.bosstarget = function()
	if cfg.target=="worldboss" then
		return true
	end
	return nil
end

cfg.health_tmp = {}
cfg.health_tmp.health=nil
cfg.health_tmp.healthmax=nil
local health_save = function()
	cfg.health_tmp.health=cfg.health
	cfg.health_tmp.healthmax=cfg.healthmax
end

local health_changed = function()
	if cfg.health~=cfg.health_tmp.health or  cfg.healthmax~=cfg.health_tmp.healthmax then
		return true
	else
		return false
	end
end

lib.UpdateHealth = function()
	health_save()
	cfg.healthmax=UnitHealthMax("target")
	cfg.health=(cfg.healthmax==0) and 100 or hedlib.round(UnitHealth("target")*100/cfg.healthmax,0)
	if health_changed() then
		cfg.Update=true
		return true
	end
end

local playerhealth,playerhealthmax,playerhealthperc
lib.UpdatePlayerHealth = function()
	playerhealth=cfg.phealth
	playerhealthmax=cfg.phealthmax
	playerhealthperc=cfg.phealthperc
	cfg.phealthmax=UnitHealthMax("player")
	cfg.phealth=UnitHealth("player")
	cfg.phealthperc=(cfg.phealthmax==0) and 100 or hedlib.round(cfg.phealth*100/cfg.phealthmax,0)
	if cfg.phealth~=playerhealth or cfg.phealthmax~=playerhealthmax or cfg.phealthperc~=playerhealthperc then
		cfg.Update=true
	end
end

lib.UpdateHaste = function()
	cfg.haste=GetCombatRatingBonus(CR_HASTE_MELEE)
end

local enabled, glyphType, glyphTooltipIndex, glyphSpell, icon
lib.HasGlyph = function(id)
	for i=1, NUM_GLYPH_SLOTS do
		enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i)
		if glyphSpell==id then return true end
	end
	return nil
end

local region,text
lib.PrintTooltipLines_helper = function(...)
    for i = 1, select("#", ...) do
        region = select(i, ...)
        if region and region:GetObjectType() == "FontString" then
            text = region:GetText() or " "
			print(region:GetName().." "..text)
        end
    end
end

lib.PrintTooltipLines = function(tooltip)
    lib.PrintTooltipLines_helper(tooltip:GetRegions())
end

lib.PrintSpellTooltip = function(spellid)
	Heddframe.Tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
	Heddframe.Tooltip:SetSpellByID(spellid)
	lib.PrintTooltipLines(Heddframe.Tooltip)
	Heddframe.Tooltip:Hide()
end

lib.IsTextInSpellTooltip = function(spellid,txt)
	Heddframe.Tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
	Heddframe.Tooltip:SetSpellByID(spellid)
	txt = hedlib.IsTextInTooltip(Heddframe.Tooltip,txt)
	Heddframe.Tooltip:Hide()
	return(txt)
end